<#
		.SYNOPSIS
			Creates new computer objects for UWWI Delegated OU customers.
			
        .DESCRIPTION
			Creates new computer objects in a specified OU in the netid.washington.edu domain and grants permissions to a specified account or group to connect a computer to each object.

		.EXAMPLE
			New-UWWIComputer -computerName "uwwiTest01" -path "OU=neb,OU=Delegated,DC=netid,dc=washington,dc=edu"
			
			Creates a new computer object named 'uwwiTest01' in the specified Delegated OU and grants the default group permissions to join a computer to that object.
			
		.EXAMPLE
			$computers = Get-Content c:\temp\newComputers.txt
			$sid = (Get-ADGroup "u_windowsInfrastructure_uwit_computerjoiners").sid
			$ouPath = "OU=Workstations,OU=uwit,OU=delegated,DC=netid,dc=washington,dc=edu"
			New-UWWIComputer -computerName $computers -SID $sid -path $ouPath
			
			Creates a new computer object for each name specified in the newComputers.txt file, in the OU specified, and grants the u_windowsinfrastructure_uwit_computerjoiners group rights to join a computer to each new object.
			
			The following rights are granted to the specified security principal on the new computer object:
			- Reset Password
			- Validated write to Service Principal Name
			- Validated write to DNS Host Name
			- Write Account Restrictions

		.PARAMETER computerName
			A string or string array of names to create
			
		.PARAMETER path
			The LDAP distinguishedName of the OU to create the new computer(s) in, for example "OU=neb,OU=Delegated,DC=netid,DC=washington,DC=edu"
			
		.PARAMETER sid
			A System.Principal.SecurityIdentifier that will be granted the rights to join a computer to this object.  By default, the group u_windowsinfrastructure_computerjoiners is used.
			
		.PARAMETER domainName
			A string containing the DNS name of the Active Directory domain.
				
        .NOTES
			Author  : James Morris - jmorris@uw.edu
			Created : 08/12/2013
			Modified: 08/28/2013 by jmorris
			
        .LINK
			UWWI Documentation
				Pending
			
		.LINK
			TechNet Extended Rights Reference
				http://technet.microsoft.com/en-us/library/ff405676.aspx
				
		.LINK
			MSDN Control Access Rights
				http://msdn.microsoft.com/en-us/library/windows/desktop/ms680945(v=vs.85).aspx
		
		.LINK
			

#>

[cmdletbinding()]
param([Parameter(Mandatory=$true)][string]$path = "ou=Unclaimed Computers,dc=netid,dc=washington,dc=edu",
	  [System.Security.Principal.SecurityIdentifier]$sid = [System.Security.Principal.SecurityIdentifier]'S-1-5-21-1478355014-127360780-1969717230-794345',
	  [Parameter(Mandatory=$true)][string[]] $computerName
	  )

# Verify the AD Module is available as it's required for the rest of the script
If (!(Get-Module ActiveDirectory)) {
	Throw "The ActiveDirectory PowerShell is not available or not installed."
}
	  
# Set constants 
$AD_RIGHTS_WRITE_PROPERTY = 'WriteProperty'
$AD_RIGHTS_SELF = 'Self'
$AD_RIGHTS_EXTENDED_RIGHT = 'ExtendedRight'
$AccessControlType = "Allow"
$domainName = "netid.washington.edu"

Function Get-DCName {
	# Get a domain controller for the specified domain and return a string containing the 
	# hostname or return NULL
	Try { (Get-ADDomainController -domain $domainName -discover).hostname[0].ToString() } Catch { $null }
}

# We need to use the same DC through the entire process to avoid replication issues.  
# Verify the DC is responding (at a basic level).  If it's not, try again (in theory this 
# should get a different DC and then terminate if that didn't succeed.
Try { 
	$dc = Get-DCName 
	if ($dc -eq $null) {
		$dc = Get-DCName
		if ($dc -eq $null) { 
			Throw "Unable to locate a domain controller for $domainName"
		}
	}
} Catch {  }

# Verify we can reach the domain controller returned above.  If not, lets get
# another one and try a final time before terminating
if (-not(Test-Connection -quiet -count 1 -computerName $dc)) {
	Try { $dc = Get-DCName } Catch {}
	if (-not(Test-Connection -quiet -count 1 -computerName $dc)) {
		Throw "Unable to locate or reach a domain controller for $domainName"
	}
}

# Verify the provided OU path is valid
Try {
	$ou = Get-ADOrganizationalUnit -server $dc -identity $path
} Catch {
	Throw "Specified OU Path does not exist or is not available"
}

# Create AD Access Rule objects to hold the rights were 
# wanting to assign the specified security principal 
$USER_ACCOUNT_RESTRICTIONS = 	New-Object System.DirectoryServices.ActiveDirectoryAccessRule ($sid, $AD_RIGHTS_WRITE_PROPERTY, $accessControlType, [GUID]'4C164200-20C0-11D0-A768-00AA006E0529')
$VALIDATED_SPN = 				New-Object System.DirectoryServices.ActiveDirectoryAccessRule ($sid, $AD_RIGHTS_SELF, 			$accessControlType, [GUID]'F3A64788-5306-11D1-A9C5-0000F80367C1')
$VALIDATED_DNS_HOST_NAME = 		New-Object System.DirectoryServices.ActiveDirectoryAccessRule ($sid, $AD_RIGHTS_SELF, 			$accessControlType, [GUID]'72E39547-7B18-11D1-ADEF-00C04FD8D5CD')
$RESET_PASSWORD = 				New-Object System.DirectoryServices.ActiveDirectoryAccessRule ($sid, $AD_RIGHTS_EXTENDED_RIGHT, $accessControlType, [GUID]'00299570-246D-11D0-A768-00AA006E0529')

foreach ($computer in $computerName) {

	# Make a computer object does not already exist.  If it does not, then 
	# create new computer object conforming to UWWI standard (cn = samAccount)
	Try { 
		Get-ADComputer -server $dc -identity $computer | out-null
		Write-Host -Fore Red "A computer object already exists for the name $computer"
	} Catch {
		Try { 
			New-ADComputer -server $dc -name $computer -samAccountName $computer -path $path
		
			Try { 
				# Get the newly created computer object, then get the ADSI incarnation 
				# of that object.  This shortcuts the need to use a DirectorySearcher and
				# DirectoryEntry object
				$adComputer = Get-ADComputer -identity $computer -server $dc
				$adsiComputer = [adsi]"LDAP://$dc/$($adComputer.distinguishedName)"
				
				# Get the security options for the object and set the mask to exclude the SACL.
				# This is necessary as only Domain Admins can write back the full security 
				# security descriptor (SACL + DACL).  Everyone else can only update the DACL.
				[System.DirectoryServices.DirectoryEntryConfiguration]$SecurityOptions = $adsiComputer.get_Options();
				$SecurityOptions.SecurityMasks = [System.DirectoryServices.SecurityMasks]'Dacl'
				
				# Add Access Rules to the DACL, commiting changes as we go.  If one or more of these
				# commits fails, then the object will be left with only the partial set.  
				$adsiComputer.get_ObjectSecurity().AddAccessRule($USER_ACCOUNT_RESTRICTIONS)
				$adsiComputer.CommitChanges()
				$adsiComputer.get_ObjectSecurity().AddAccessRule($VALIDATED_SPN)
				$adsiComputer.CommitChanges()
				$adsiComputer.get_ObjectSecurity().AddAccessRule($VALIDATED_DNS_HOST_NAME)
				$adsiComputer.CommitChanges()
				$adsiComputer.get_ObjectSecurity().AddAccessRule($RESET_PASSWORD)
				$adsiComputer.CommitChanges()
				
				write-Host -fore Green "New computer objected created for $computer"
			} Catch {
				# Write an error to the console if the ACL update fails for any reason
				Write-host -fore Red "Unable to update ACL for $computer"
			} 
		} catch { 
			# If the creation fails, we don't want to skip any other computers that might be in the 
			# passed array, so write an error to the console and move on.  
			Write-Host -fore Red "Unable to create a new object for $computer"
		}
	} 
} 

# SIG # Begin signature block
# MIIOdgYJKoZIhvcNAQcCoIIOZzCCDmMCAQExCzAJBgUrDgMCGgUAMGkGCisGAQQB
# gjcCAQSgWzBZMDQGCisGAQQBgjcCAR4wJgIDAQAABBAfzDtgWUsITrck0sYpfvNR
# AgEAAgEAAgEAAgEAAgEAMCEwCQYFKw4DAhoFAAQUer2AnHLYJrsCFKpY/pq3LnDM
# Su+ggglbMIIEkzCCA3ugAwIBAgIQR4qO+1nh2D8M4ULSoocHvjANBgkqhkiG9w0B
# AQUFADCBlTELMAkGA1UEBhMCVVMxCzAJBgNVBAgTAlVUMRcwFQYDVQQHEw5TYWx0
# IExha2UgQ2l0eTEeMBwGA1UEChMVVGhlIFVTRVJUUlVTVCBOZXR3b3JrMSEwHwYD
# VQQLExhodHRwOi8vd3d3LnVzZXJ0cnVzdC5jb20xHTAbBgNVBAMTFFVUTi1VU0VS
# Rmlyc3QtT2JqZWN0MB4XDTEwMDUxMDAwMDAwMFoXDTE1MDUxMDIzNTk1OVowfjEL
# MAkGA1UEBhMCR0IxGzAZBgNVBAgTEkdyZWF0ZXIgTWFuY2hlc3RlcjEQMA4GA1UE
# BxMHU2FsZm9yZDEaMBgGA1UEChMRQ09NT0RPIENBIExpbWl0ZWQxJDAiBgNVBAMT
# G0NPTU9ETyBUaW1lIFN0YW1waW5nIFNpZ25lcjCCASIwDQYJKoZIhvcNAQEBBQAD
# ggEPADCCAQoCggEBALw1oDZwIoERw7KDudMoxjbNJWupe7Ic9ptRnO819O0Ijl44
# CPh3PApC4PNw3KPXyvVMC8//IpwKfmjWCaIqhHumnbSpwTPi7x8XSMo6zUbmxap3
# veN3mvpHU0AoWUOT8aSB6u+AtU+nCM66brzKdgyXZFmGJLs9gpCoVbGS06CnBayf
# UyUIEEeZzZjeaOW0UHijrwHMWUNY5HZufqzH4p4fT7BHLcgMo0kngHWMuwaRZQ+Q
# m/S60YHIXGrsFOklCb8jFvSVRkBAIbuDlv2GH3rIDRCOovgZB1h/n703AmDypOmd
# RD8wBeSncJlRmugX8VXKsmGJZUanavJYRn6qoAcCAwEAAaOB9DCB8TAfBgNVHSME
# GDAWgBTa7WR0FJwUPKvdmam9WyhNizzJ2DAdBgNVHQ4EFgQULi2wCkRK04fAAgfO
# l31QYiD9D4MwDgYDVR0PAQH/BAQDAgbAMAwGA1UdEwEB/wQCMAAwFgYDVR0lAQH/
# BAwwCgYIKwYBBQUHAwgwQgYDVR0fBDswOTA3oDWgM4YxaHR0cDovL2NybC51c2Vy
# dHJ1c3QuY29tL1VUTi1VU0VSRmlyc3QtT2JqZWN0LmNybDA1BggrBgEFBQcBAQQp
# MCcwJQYIKwYBBQUHMAGGGWh0dHA6Ly9vY3NwLnVzZXJ0cnVzdC5jb20wDQYJKoZI
# hvcNAQEFBQADggEBAMj7Y/gLdXUsOvHyE6cttqManK0BB9M0jnfgwm6uAl1IT6TS
# IbY2/So1Q3xr34CHCxXwdjIAtM61Z6QvLyAbnFSegz8fXxSVYoIPIkEiH3Cz8/dC
# 3mxRzUv4IaybO4yx5eYoj84qivmqUk2MW3e6TVpY27tqBMxSHp3iKDcOu+cOkcf4
# 2/GBmOvNN7MOq2XTYuw6pXbrE6g1k8kuCgHswOjMPX626+LB7NMUkoJmh1Dc/VCX
# rLNKdnMGxIYROrNfQwRSb+qz0HQ2TMrxG3mEN3BjrXS5qg7zmLCGCOvb4B+MEPI5
# ZJuuTwoskopPGLWR5Y0ak18frvGm8C6X0NL2KzwwggTAMIIEKaADAgECAgImFDAN
# BgkqhkiG9w0BAQUFADCBlDELMAkGA1UEBhMCVVMxCzAJBgNVBAgTAldBMSEwHwYD
# VQQKExhVbml2ZXJzaXR5IG9mIFdhc2hpbmd0b24xFDASBgNVBAsTC1VXIFNlcnZp
# Y2VzMRcwFQYDVQQDEw5VVyBTZXJ2aWNlcyBDQTEmMCQGCSqGSIb3DQEJARYXaGVs
# cEBjYWMud2FzaGluZ3Rvbi5lZHUwHhcNMTIwODE0MTYwMzEyWhcNMTUwODE1MTYw
# MzEyWjCBujEhMB8GA1UEChMYVW5pdmVyc2l0eSBvZiBXYXNoaW5ndG9uMR8wHQYD
# VQQLExZJbmZvcm1hdGlvbiBUZWNobm9sb2d5MRwwGgYJKoZIhvcNAQkBFg1jaS1p
# YW1AdXcuZWR1MRAwDgYDVQQHEwdTZWF0dGxlMRMwEQYDVQQIEwpXYXNoaW5ndG9u
# MQswCQYDVQQGEwJVUzEiMCAGA1UEAxMZY29kZS5uZXRpZC53YXNoaW5ndG9uLmVk
# dTCCASIwDQYJKoZIhvcNAQEBBQADggEPADCCAQoCggEBALmVM514ZAcjgZn+Oa7j
# ZppF092FFNSajhHwfO3FWa4e140z7rDvUlXQ++Qg8J7ANzqLTC8gdQ8oLXqd0StL
# 0ZKTfyXXnJAoXoPMcK8GKqIG+fKBKeKQlFuA3IMwF8BfDCr9GPnvuL7BPiRBfP5o
# d3IyR4OgeRWH4BeN8LOrusCE5LqnJquUnSLKCEYqmtLkU0ykD55Jk3pjyXjqkFwD
# 8CQWdzU6k3apeJKijLS0Lw5zgbSkOKV8NnL7tYbT4Qo9QvR7+u7KJety7QonOGsR
# nGlK0wrOZGJlMV5gaH6s19DlNPTT9U3pfacXt4ARvYjtx8MgwTap94Y9HEi0JJJN
# 2mECAwEAAaOCAXMwggFvMAwGA1UdEwEB/wQCMAAwEwYDVR0lBAwwCgYIKwYBBQUH
# AwMwHQYDVR0OBBYEFPBMWCTVBRKGAJOIsUoxlL+D1CSiMCQGA1UdEQQdMBuCGWNv
# ZGUubmV0aWQud2FzaGluZ3Rvbi5lZHUwgcEGA1UdIwSBuTCBtoAUVdfBM8b6k/gn
# PcsgS/VajliXfXShgZqkgZcwgZQxCzAJBgNVBAYTAlVTMQswCQYDVQQIEwJXQTEh
# MB8GA1UEChMYVW5pdmVyc2l0eSBvZiBXYXNoaW5ndG9uMRQwEgYDVQQLEwtVVyBT
# ZXJ2aWNlczEXMBUGA1UEAxMOVVcgU2VydmljZXMgQ0ExJjAkBgkqhkiG9w0BCQEW
# F2hlbHBAY2FjLndhc2hpbmd0b24uZWR1ggEAMEEGA1UdHwQ6MDgwNqA0oDKGMGh0
# dHA6Ly9jZXJ0cy5jYWMud2FzaGluZ3Rvbi5lZHUvVVdTZXJ2aWNlc0NBLmNybDAN
# BgkqhkiG9w0BAQUFAAOBgQA0WGRa2o6SOfc/m8drwY/OqItoaWCGtzpnVhC61vEt
# NZ7sHV6IFdLYwnbudQX2CcwGE2uOYZt2LqbXVNpi5m4ZHoWqQXCCpmqueT79b3+g
# OUuto1sPpI2KwV6TbI1/8xqX9Vpvx8ZazGtc08G2QLowbDkXIWJ1/6q0b144aFCU
# WDGCBIUwggSBAgEBMIGbMIGUMQswCQYDVQQGEwJVUzELMAkGA1UECBMCV0ExITAf
# BgNVBAoTGFVuaXZlcnNpdHkgb2YgV2FzaGluZ3RvbjEUMBIGA1UECxMLVVcgU2Vy
# dmljZXMxFzAVBgNVBAMTDlVXIFNlcnZpY2VzIENBMSYwJAYJKoZIhvcNAQkBFhdo
# ZWxwQGNhYy53YXNoaW5ndG9uLmVkdQICJhQwCQYFKw4DAhoFAKB4MBgGCisGAQQB
# gjcCAQwxCjAIoAKAAKECgAAwGQYJKoZIhvcNAQkDMQwGCisGAQQBgjcCAQQwHAYK
# KwYBBAGCNwIBCzEOMAwGCisGAQQBgjcCARUwIwYJKoZIhvcNAQkEMRYEFHQKSV9c
# YWxcJ0tCRqcWP51xxRBAMA0GCSqGSIb3DQEBAQUABIIBALbplx6wtEh2dm8IaZEM
# KUjnukNj8KI9TLRoh+yu11Y0tDZepoxy4Gisd1IbP7LdzseW2ZqztZxchFnOwnaX
# vVaJ8rgfRbxKkAI0WvfEtEbmm3T+vgHzZjoiwSTWV0AUGEi1oNPLj9D09/2IXdJW
# tjz43psjqlOCA/Nr6mT+MIk8RTb7tG+HbjSmRD4C9hvUx3ljnOqJsB0l58pDbnQT
# 8FBycemSG2jl2LWrmghK6NAUB2zaUgFV/J38DzhmY8hw8YkQ9pGCDWl3j+Ly9krP
# uBFVpNyUNwslbip0fIiyay66yEg12DKsWHwOsUDINtRfFS95UCpno6t3mXkWlxum
# XMGhggJEMIICQAYJKoZIhvcNAQkGMYICMTCCAi0CAQAwgaowgZUxCzAJBgNVBAYT
# AlVTMQswCQYDVQQIEwJVVDEXMBUGA1UEBxMOU2FsdCBMYWtlIENpdHkxHjAcBgNV
# BAoTFVRoZSBVU0VSVFJVU1QgTmV0d29yazEhMB8GA1UECxMYaHR0cDovL3d3dy51
# c2VydHJ1c3QuY29tMR0wGwYDVQQDExRVVE4tVVNFUkZpcnN0LU9iamVjdAIQR4qO
# +1nh2D8M4ULSoocHvjAJBgUrDgMCGgUAoF0wGAYJKoZIhvcNAQkDMQsGCSqGSIb3
# DQEHATAcBgkqhkiG9w0BCQUxDxcNMTMwODI5MDEyNjAyWjAjBgkqhkiG9w0BCQQx
# FgQUO6NqxZlE/E+Hq4tQPZyZIRyPzrcwDQYJKoZIhvcNAQEBBQAEggEAERhLcZk+
# 6mUSg7GOSNK9qLgLf5H/sb28BSRSBrrGv3FiLhf/1jPQdxBQ9xpaukdA1Od3UGHw
# /rdwT8/3vqIb/AzIyw+WdVeE5pz50g/Dx3R0BMy0VV3zeYSL0SN9Ag8T8N7ij9CQ
# RDVDLduGyXhOWW56G5tTjAWKzPTcO1ObnyHt/3xSjI3bOdJbTj7m1fRdk6N4KW/3
# IX/w+dIOgaYMtp7/Co8nuoxNR+nA3ibNQc9VmVSDMbD/FfQVal2Jzma1qVq0YgdG
# zaIacVlpYddXHLlg8yWtAMjekqj0nTZT/DHV5wcqOSK7pF3QwUvn8R8DJk3D1NbM
# pyxfkDV9oDZBmQ==
# SIG # End signature block
